// (c) Copyright Microsoft Corporation.
// This source is subject to [###LICENSE_NAME###].
// Please see [###LICENSE_LINK###] for details.
// All other rights reserved.
//
// <autogenerated />
// The Visual Studio metadata is not subjected to same source analysis

//*****************************************************************************
// Attributes for tests that are hosted in a web server (ASP.NET).
// Copyright(c) Microsoft Corporation, 2004
//*****************************************************************************

using System.Diagnostics.CodeAnalysis;

// Suppress AvoidNamespacesWithFewTypes because we want a separate namespace for the ASP.NET stuff.
[module: SuppressMessage("Microsoft.Design", "CA1020:AvoidNamespacesWithFewTypes", Scope = "namespace", Target = "Microsoft.VisualStudio.TestTools.UnitTesting.Web")]

namespace Microsoft.VisualStudio.TestTools.UnitTesting.Web
{
    using System;

    /// <summary>
    /// UrlToTest specifies the url that should be requested to give context to the test.
    /// </summary>
    [AttributeUsage(AttributeTargets.Method, AllowMultiple = false)]
    public sealed class UrlToTestAttribute: Attribute
    {
        // Suppress UriParametersShouldNotBeStrings because this is storage
        // for a url entered by the user that may or may not be completely valid
        // at all times. A Uri must be valid in order to be constructed.
        /// <summary>
        /// An unused attribute.
        /// </summary>
        /// <param name="urlToTest"></param>
        [SuppressMessage("Microsoft.Design", "CA1054:UriParametersShouldNotBeStrings")]
        public UrlToTestAttribute(string urlToTest)
        {
            m_urlToTest = urlToTest;
        }

        // Suppress UriPropertiesShouldNotBeStrings because this is storage
        // for a url entered by the user that may or may not be completely valid
        // at all times. A Uri must be valid in order to be constructed.
        /// <summary>
        /// Gets the URL to test.
        /// </summary>
        [SuppressMessage("Microsoft.Design", "CA1056:UriPropertiesShouldNotBeStrings")]
        public string UrlToTest
        {
            get { return m_urlToTest; }
        }

        private string m_urlToTest;
    }

    /// <summary>
    /// AspNetDevelopmentServer specifies the settings to be used for the
    /// ASP.NET Development Server for the test.
    /// </summary>
    [AttributeUsage(AttributeTargets.Method, AllowMultiple = true)]
    public sealed class AspNetDevelopmentServerAttribute: Attribute
    {
        /// <summary>
        /// Initializes the AspNetDevelopmentServerAttribute.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="pathToWebApp">The path to web app.</param>
        public AspNetDevelopmentServerAttribute(string name, string pathToWebApp)
        : this(name, pathToWebApp, null)
        {
        }

        /// <summary>
        /// Initializes the AspNetDevelopmentServerAttribute.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="pathToWebApp">The path to web app.</param>
        /// <param name="webAppRoot">The web app root.</param>
        public AspNetDevelopmentServerAttribute(string name, string pathToWebApp, string webAppRoot)
        {
            m_pathToWebApp = pathToWebApp;
            m_webAppRoot = webAppRoot;
            m_name = name;
        }

        /// <summary>
        /// Gets the name.
        /// </summary>
        public string Name
        {
            get { return m_name; }
        }

        /// <summary>
        /// Gets the PathToWebApp.
        /// </summary>
        public string PathToWebApp
        {
            get { return m_pathToWebApp; }
        }

        /// <summary>
        /// Gets the web app root.
        /// </summary>
        public string WebAppRoot
        {
            get { return m_webAppRoot; }
        }

        private string m_name;
        private string m_pathToWebApp;
        private string m_webAppRoot;
    }

    /// <summary>
    /// AspNetDevelopmentServerHost specifies the settings to be used when
    /// ASP.NET Development Server is the host server for the test.
    /// </summary>
    [AttributeUsage(AttributeTargets.Method, AllowMultiple = false)]
    public sealed class AspNetDevelopmentServerHostAttribute: Attribute
    {
        /// <summary>
        /// Initializes the AspNetDevelopmentServerHostAttribute.
        /// </summary>
        /// <param name="pathToWebApp">The path to the web app.</param>
        public AspNetDevelopmentServerHostAttribute(string pathToWebApp)
        : this(pathToWebApp, null)
        {
        }

        /// <summary>
        /// Initializes the AspNetDevelopmentServerHostAttribute.
        /// </summary>
        /// <param name="pathToWebApp">The path to the web app.</param>
        /// <param name="webAppRoot">The web app root.</param>
        public AspNetDevelopmentServerHostAttribute(string pathToWebApp, string webAppRoot)
        {
            m_pathToWebApp = pathToWebApp;
            m_webAppRoot = webAppRoot;
        }

        /// <summary>
        /// Gets the path to the web application.
        /// </summary>
        public string PathToWebApp
        {
            get { return m_pathToWebApp; }
        }

        /// <summary>
        /// Gets the WebAppRoot.
        /// </summary>
        public string WebAppRoot
        {
            get { return m_webAppRoot; }
        }

        private string m_pathToWebApp;
        private string m_webAppRoot;
    }

    /// <summary>
    /// For ASP.NET sites that require basic authentication, specify the
    /// user name and password using the Credential attribute.
    /// WARNING: The password is stored in plain text in source code
    /// and in the compiled assembly. Restrict access to the source code
    /// and assembly to protect this sensitive information.
    /// </summary>
    [AttributeUsage(AttributeTargets.Method, AllowMultiple = false)]
    public sealed class CredentialAttribute : Attribute
    {
        /// <summary>
        /// Specify the user name and password needed to access the web site under test.
        /// </summary>
        /// <param name="userName">The user name.</param>
        /// <param name="password">
        /// The password. WARNING: The password is stored in plain text in source code
        /// and in the compiled assembly. Restrict access to the source code and assembly
        /// to protect this sensitive information.
        /// </param>
        public CredentialAttribute(string userName, string password)
        {
            m_userName = userName;
            m_password = password;
        }

        /// <summary>
        /// Specify the user name, password, and domain needed to access the web site under test.
        /// </summary>
        /// <param name="userName">The user name.</param>
        /// <param name="password">
        /// The password. WARNING: The password is stored in plain text in source code
        /// and in the compiled assembly. Restrict access to the source code and assembly
        /// to protect this sensitive information.
        /// </param>
        /// <param name="domain">The domain.</param>
        public CredentialAttribute(string userName, string password, string domain)
        : this(userName, password)
        {
            m_domain = domain;
        }

        /// <summary>
        /// Gets the user name.
        /// </summary>
        public string UserName
        {
            get { return m_userName; }
        }

        /// <summary>
        /// Gets the password.
        /// </summary>
        public string Password
        {
            get { return m_password; }
        }

        /// <summary>
        /// Gets the domain.
        /// </summary>
        public string Domain
        {
            get { return m_domain; }
        }

        private string m_userName;
        private string m_password;
        private string m_domain = string.Empty;
    }
}